// menu.cc
// version 2.0, Lg, July 9, 2017
//              history: version 1.0, Lg, July 17, 1995
//                       version 1.1, Lg, Aug. 2, 1996
//                       version 1.6, PS, Sept 2011


#include <string.h>
#include "menu.h"
#include "uti1a.h"
#include "lineInput.h"


static char* mid = (char*)"menu.cc";


void menu::def( const char* name, proc action, const char* comment, 
/*         ===  */                                     int invisible ) 
{
    int status;
    simpleStringListEntry* atp = retrieve( name, &status );
    if ( status == exact_match ) 
        warning( mid, "ignoring attempt to redefine command ", name );
    menu_item* itemp = new menu_item;
    if ( itemp==0 ) memory_error();
    itemp->strngp = name;
    itemp->_action = action;
    itemp->_comment = comment;
    itemp->_invisible = invisible;
    insert( itemp, atp );
}


int menu::execute_command( const char* name )
{ //      ===============
    int status; 
    simpleStringListEntry* atp = retrieve( name, &status );
    if ( status == no_match ) {
        input_error( *_linecountp, "unknown command ", name );
        return 0;
    }
    menu_item* mip = (menu_item*) atp; 
    if ( status == is_prefix ) mip = (menu_item*) succ(mip);
    mip->_action();
    return 1;
}


static int is_legal_char( char ch )
{ //       -------------
    return isalnum(ch) || (ch=='_') || (ch=='\\');
}


int menu::get_and_execute_command( istream& is )
{ //      =======================
    const int bufsize=80;
    char buf[bufsize];
    li_eatspace(is);
    int nchars = 
    li_get_string( is, buf, bufsize, &is_legal_char, (int)*_linecountp );
    if ( (nchars==0) || (nchars>=bufsize) ) return 0;
    return execute_command(buf);
}


void menu::print_command_list( ostream& os, int mode, int tab )
{ //       ==================
    simpleListEntry* p = _first; 
    menu_item* mip;
    int space;
    while (p) {
        mip = (menu_item*) p; 
        p = p->_next;
        if ( mip->_invisible ) continue;
        os << mip->strngp; 
        if ( mode==1 ) {
            space = (tab-3) - (int)strlen( mip->strngp );
            while ( space-- > 0 ) os << ' ';
            os << mip->_comment << endl;
        }
        else os << ' ';
    }
    if ( mode==0 ) os << endl;
}
